module Ulule

  class Client

    attr_accessor :username, :key

    def initialize(options = {})

      @key = options.fetch(:key) { ENV.fetch('ULULE_API_KEY') }
      @username = options.fetch(:username) { ENV.fetch('ULULE_USERNAME') }
      @host = options.fetch(:host) { 'https://api.ulule.com/v1/' }
      @projects = 'projects'
      @comments = 'comments'
      @supporters = 'supporters'
      @news = 'news'
      @users = 'users'

    end

    def me
      get("#{@users}/#{@username}")
    end

    def my_projects(options = {})
      get("#{@users}/#{@username}/#{@projects}", options)
    end

    def projects(user, options = {})
      get("#{@users}/#{user}/#{@projects}", options)
    end

    def project slug
      get("#{@projects}/#{slug}")
    end

    def project_comments(slug, options = {})
      get("#{@projects}/#{slug}/#{@comments}", options)
    end

    def project_news slug
      get("#{@projects}/#{slug}/#{@news}")
    end

    def news id
      get("#{@news}/#{id}")
    end

    def news_comments(id, options = {})
      get("#{@news}/#{id}/#{@comments}", options)
    end

    def supporters slug
      get ("#{@projects}/#{slug}/#{@supporters}")
    end

   private

    def get(path, params = {})
      uri = URI "#{@host}#{path}#{build_params params}"
      req = Net::HTTP::Get.new uri
      req['Authorization'] = "ApiKey #{username}:#{key}"
      parse Net::HTTP.start(uri.hostname, uri.port, use_ssl: true) { |http| http.request(req) }
    end

    def parse http_response
      case http_response
        when Net::HTTPSuccess
          JSON.parse http_response.body
        when Net::HTTPUnauthorized
          raise AuthenticationError
        else
          raise Error, "Unexpected HTTP response, code #{http_response.code}"
      end
    end

    def build_params params
      str = nil
      params.each_key do |key|
        str = "#{str}#{CGI::escape(key.to_s)}=#{CGI::escape(params[key])}&" unless params[key].empty?
      end
       str.nil? ? '' : "?#{str[0...-1]}"
    end

  end

end